from strategy_field.registry import Registry

from .base import Exporter
from .csv import ExportAsCSV
from .txt import ExportAsText

registry = Registry(Exporter)
registry.register(ExportAsText)
registry.register(ExportAsCSV)



import pytest
from demo.factories import user_grant_permission
from django.urls import reverse

pytestmark = [pytest.mark.admin]


@pytest.fixture()
def category():
    from demo.factories import CategoryFactory

    return CategoryFactory()


def test_changelist(django_app, std_user, category):
    url = reverse("admin:hope_smart_export_category_changelist")

    with user_grant_permission(std_user, ["hope_smart_export.view_category"]):
        assert std_user.has_perm("hope_smart_export.view_category")
        res = django_app.get(url, user=std_user)
    assert res.status_code == 200


def test_change(django_app, std_user, category):
    url = reverse("admin:hope_smart_export_category_change", args=(category.id,))
    with user_grant_permission(std_user, ["hope_smart_export.change_category"]):
        res = django_app.get(url, user=std_user)
    assert res.status_code == 200


def test_add(django_app, std_user, category):
    url = reverse("admin:hope_smart_export_category_add")
    with user_grant_permission(std_user, ["hope_smart_export.add_category"]):
        res = django_app.get(url, user=std_user)
    assert res.status_code == 200



from typing import TYPE_CHECKING
from unittest import mock

import openpyxl
import pytest
from strategy_field.utils import fqn

from hope_smart_export.exporters.xls import ExportAsXls

if TYPE_CHECKING:
    from django.contrib.auth.models import User
    from openpyxl.workbook.workbook import Workbook
    from openpyxl.worksheet.worksheet import Worksheet

    from hope_smart_export.models import Configuration


@pytest.fixture()
def user():
    from demo.factories import UserFactory

    return UserFactory()


@pytest.fixture()
def cfg(db):
    from demo.factories import ConfigurationFactory
    from django.contrib.auth.models import User
    from django.contrib.contenttypes.models import ContentType

    return ConfigurationFactory(
        content_type=ContentType.objects.get_for_model(User),
        columns="username\nemail",
        exporter=fqn(ExportAsXls),
        option="T",
        data={"sheet_name": "TestSheet #1"},
    )


@pytest.mark.parametrize(
    "line",
    [
        "username\nemail",
        "{{record.username}}\nemail",
        "{{record.username}}\n{{record.email}}",
        "{{record.username}}  \n   {{record.email}}   ",
        "{{record.username}}\n{{record.email}}",
    ],
)
def test_export(db, line, cfg: "Configuration", user: "User"):
    from django.contrib.auth.models import User

    with mock.patch.object(cfg, "columns", f"#comment\n{line}"):
        assert len(cfg.get_processor().columns) == 2
        data = cfg.export(User.objects.all())
        wb = openpyxl.load_workbook(data)
        assert wb


def test_export_headers(cfg: "Configuration", user: "User"):
    from django.contrib.auth.models import User

    with mock.patch.object(cfg, "headers", "username\nemail"):
        data = cfg.export(User.objects.all())
        wb: "Workbook" = openpyxl.load_workbook(data)
        sh: "Worksheet" = wb.worksheets[0]
        assert sh.title == "TestSheet #1"
        assert [c.value for c in list(sh.rows)[0]] == ["#", "username", "email"]



# Generated by Django 5.1.1 on 2024-10-03 07:03

import django.db.models.deletion
import strategy_field.fields
from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ("contenttypes", "0002_remove_content_type_name"),
    ]

    operations = [
        migrations.CreateModel(
            name="Category",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=255, unique=True)),
                ("code", models.SlugField(max_length=255, unique=True)),
            ],
        ),
        migrations.CreateModel(
            name="Configuration",
            fields=[
                ("id", models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name="ID")),
                ("name", models.CharField(max_length=255, unique=True)),
                ("code", models.SlugField(max_length=255, unique=True)),
                ("columns", models.TextField(default="")),
                ("exporter", strategy_field.fields.StrategyField()),
                ("headers", models.TextField(blank=True, default="")),
                ("data", models.JSONField(blank=True, default={})),
                ("enabled", models.BooleanField(blank=True, default=False)),
                (
                    "option",
                    models.CharField(choices=[("M", "Memory Safe"), ("T", "Fastest")], default="T", max_length=1),
                ),
                (
                    "max_queries",
                    models.IntegerField(
                        blank=True,
                        default=0,
                        help_text="Max queries expected for this template. Log warning otherwise.",
                        null=True,
                    ),
                ),
                ("categories", models.ManyToManyField(blank=True, to="hope_smart_export.category")),
                (
                    "content_type",
                    models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to="contenttypes.contenttype"),
                ),
            ],
        ),
    ]



import io
from typing import TYPE_CHECKING, Any, Optional

from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError
from django.db import connections, models
from django.db.models import Model, QuerySet
from django.template import Context, Template, TemplateSyntaxError
from django.test.utils import CaptureQueriesContext
from django.utils.functional import cached_property
from strategy_field.fields import StrategyField

from hope_smart_export.exporters.registry import registry
from hope_smart_export.utils import chunked_iterator

if TYPE_CHECKING:
    from hope_smart_export.exporters import Exporter


class Processor:
    def __init__(self, cfg: "Configuration"):
        self.cfg = cfg
        self._columns: Optional[list[Template]] = None

    @cached_property
    def headers(self) -> list[str]:
        if self.cfg.headers:
            defined_headers = str(self.cfg.headers).split("\n")
            headers_len = len(defined_headers)
            columns_len = len(self.columns)
            for i in range(headers_len, columns_len):
                defined_headers.append("-")
            return defined_headers[:columns_len]
        return []

    @cached_property
    def columns(self) -> list[Template]:
        self._columns = []
        for config_line in self.cfg.columns.split("\n"):
            raw_line = config_line.strip()
            if raw_line.startswith("#"):
                continue
            elif raw_line.startswith("{{"):
                line = raw_line
            elif raw_line.startswith("{%"):
                line = raw_line
            elif raw_line:
                if "record." not in raw_line:
                    line = "{{record.%s}}" % raw_line
                else:
                    line = "{{%s}}}" % raw_line
            else:
                raise ValidationError(f"Unexpected format: '{raw_line}'")
            try:
                tpl = Template(line)
                tpl.render(Context({}))
                self._columns.append(tpl)
            except TemplateSyntaxError:
                raise ValidationError(f"Invalid synthax: {line}")
        return self._columns

    def get_row_values(self, record: "Any"):
        return [column.render(Context({"record": record})) for column in self.columns]


class ConfigurationQuerySet(models.QuerySet["AnyModel"]):
    pass


class ConfigurationManager(models.Manager["AnyModel"]):
    _queryset_class = ConfigurationQuerySet


class Category(models.Model):
    name = models.CharField(max_length=255, unique=True)
    code = models.SlugField(max_length=255, unique=True)

    class Meta:
        verbose_name_plural = "Categories"

    def __str__(self):
        return self.name


class Configuration(models.Model):
    class Option(models.TextChoices):
        MEMORY = "M", "Memory Safe"
        TIME = "T", "Fastest"

    name = models.CharField(max_length=255, unique=True)
    code = models.SlugField(max_length=255, unique=True)
    content_type = models.ForeignKey(ContentType, on_delete=models.CASCADE)
    columns = models.TextField(default="", blank=False)
    exporter: "Exporter" = StrategyField(registry=registry)
    headers = models.TextField(default="", blank=True, null=False)
    data = models.JSONField(default={}, blank=True)
    enabled = models.BooleanField(default=False, blank=True)
    categories = models.ManyToManyField(Category, blank=True)

    option = models.CharField(max_length=1, choices=Option.choices, default=Option.TIME)
    max_queries = models.IntegerField(
        default=0, blank=True, null=True, help_text="Max queries expected for this template. Log warning otherwise."
    )

    objects = ConfigurationManager

    def __str__(self) -> str:
        return str(self.name)

    def get_config(self) -> dict[str, Any]:
        return {**self.exporter.config_class.defaults, **self.data}

    def clean(self) -> None:
        self.get_processor().columns  # noqa

    def export(self, queryset: QuerySet[Model]) -> io.BytesIO | io.StringIO:
        # exporter: type[Exporter] = registry[fmt]
        if self.option == Configuration.Option.MEMORY:
            data = self.exporter.export(chunked_iterator(queryset))
        else:
            data = self.exporter.export(queryset.iterator())
        return data

    def inspect(self, queryset: QuerySet[Model], max_lines=10) -> CaptureQueriesContext:
        columns = self.get_processor().columns
        with CaptureQueriesContext(connections[queryset.db]) as ctx:
            for i, record in enumerate(queryset):
                for column in columns:
                    column.render(Context({"record": record}))
                if i >= max_lines:
                    break
        return ctx

    def get_processor(self) -> Processor:
        return Processor(self)



import abc
import io
from typing import TYPE_CHECKING, Any

from django import forms
from django.db.models import Model, QuerySet

if TYPE_CHECKING:
    from ..models import Configuration


class ExporterConfig(forms.Form):
    defaults: dict[str, Any] = {}


class Exporter(abc.ABC):
    config_class = ExporterConfig

    def __init__(self, cfg: "Configuration"):
        self.config = cfg
        super().__init__()

    @classmethod
    def name(cls):
        return cls.__name__

    @abc.abstractmethod
    def export(self, queryset: QuerySet[Model]) -> io.BytesIO | io.StringIO | str | bytes:
        pass



import io
from typing import TYPE_CHECKING

from django import forms
from django.db.models import Model, QuerySet

from .base import Exporter, ExporterConfig

if TYPE_CHECKING:
    from ..models import Processor


class TxtExporterConfig(ExporterConfig):
    field_separator = forms.CharField(max_length=1)


class ExportAsText(Exporter):
    config_class = TxtExporterConfig

    def export(self, queryset: QuerySet[Model]) -> io.BytesIO | io.StringIO:
        output = io.StringIO()
        processor: "Processor" = self.config.get_processor()
        # columns = self.config.parse_simple_config()
        for record in queryset:
            values = processor.get_row_values(record)
            output.write(self.config.data["field_separator"].join(values) + self.config.data["field_separator"])
            # for column in columns:
            #     col_value = column.render(Context({"record": record}))
            # output.write(f"{col_value}{self.config.data['field_separator']}")
            output.write("\n")
        output.seek(0)
        return output



from admin_extra_buttons.decorators import button
from admin_extra_buttons.mixins import ExtraButtonsMixin
from django import forms
from django.contrib import admin
from django.contrib.admin.helpers import AdminForm
from django.contrib.contenttypes.models import ContentType
from django.core.validators import MaxValueValidator, MinValueValidator
from django.http import HttpResponse, HttpResponseRedirect
from django.template.response import TemplateResponse

from .models import Category, Configuration


class ExportTestForm(forms.Form):
    max_records = forms.IntegerField(validators=[MinValueValidator(1), MaxValueValidator(100)], initial=10)
    content_type = forms.ModelChoiceField(queryset=ContentType.objects.all())


@admin.register(Category)
class CategoryModelAdmin(ExtraButtonsMixin, admin.ModelAdmin):
    list_display = ("name", "code")
    search_fields = ("name", "code")


@admin.register(Configuration)
class ExportConfigModelAdmin(ExtraButtonsMixin, admin.ModelAdmin):
    list_display = ("name", "code", "content_type")
    list_filter = ("content_type", "categories")
    search_fields = ("name", "code")
    exclude = ("data",)
    filter_horizontal = ("categories",)

    def get_queryset(self, request):
        return super().get_queryset(request).select_related("content_type")

    def get_changeform_initial_data(self, request):
        return {"columns": "id\n"}

    @button(visible=lambda o: o.context["original"].pk)
    def configure(self, request, pk: str) -> TemplateResponse | HttpResponseRedirect:
        ctx = self.get_common_context(request, pk)
        obj: Configuration = ctx["original"]
        form_class = obj.exporter.config_class
        status_code = 200
        if request.method == "POST":
            form = form_class(request.POST, request.FILES)
            if form.is_valid():
                obj.data = form.cleaned_data
                obj.save()
                self.message_user(request, "Configuration saved")
                return HttpResponseRedirect("..")
            status_code = 400
        else:
            form = form_class()
        fs = (("", {"fields": form_class.declared_fields}),)
        ctx["admin_form"] = AdminForm(form, fs, {})  # type: ignore[arg-type]

        ctx["form"] = form
        return TemplateResponse(request, "admin/hope_smart_export/configure.html", ctx, status=status_code)

    @button()
    def test(self, request, pk: str) -> TemplateResponse | HttpResponse:
        ctx = self.get_common_context(request, pk)
        configuration: Configuration = ctx["original"]
        status_code = 200
        if request.method == "POST":
            form = ExportTestForm(request.POST, request.FILES)
            if form.is_valid():
                qs = form.cleaned_data["content_type"].model_class().objects.all()[: form.cleaned_data["max_records"]]
                output = configuration.export(qs)
                return HttpResponse(output, content_type="text/plain")
            status_code = 400
        else:
            form = ExportTestForm(initial={"content_type": configuration.content_type})
        ctx["form"] = form
        return TemplateResponse(request, "admin/hope_smart_export/test.html", ctx, status=status_code)



import csv
import io
from typing import TYPE_CHECKING

from django import forms
from django.db.models import Model, QuerySet
from django.utils.translation import gettext as _

from .base import Exporter, ExporterConfig

if TYPE_CHECKING:
    from ..models import Processor

delimiters = ",;|:"
quotes = "'\"`"
escapechars = " \\"


class TxtExporterConfig(ExporterConfig):
    defaults = {"delimiter": ",", "quotechar": "'", "quoting": csv.QUOTE_ALL, "escapechar": ""}
    delimiter = forms.ChoiceField(label=_("Delimiter"), choices=list(zip(delimiters, delimiters)), initial=",")
    quotechar = forms.ChoiceField(label=_("Quotechar"), choices=list(zip(quotes, quotes)), initial="'")
    quoting = forms.TypedChoiceField(
        coerce=int,
        label=_("Quoting"),
        choices=(
            (csv.QUOTE_ALL, _("All")),
            (csv.QUOTE_MINIMAL, _("Minimal")),
            (csv.QUOTE_NONE, _("None")),
            (csv.QUOTE_NONNUMERIC, _("Non Numeric")),
        ),
        initial=csv.QUOTE_ALL,
    )

    escapechar = forms.ChoiceField(label=_("Escapechar"), choices=(("", ""), ("\\", "\\")), required=False)


class ExportAsCSV(Exporter):
    config_class = TxtExporterConfig

    def export(self, queryset: QuerySet[Model]) -> io.BytesIO | io.StringIO:
        config = self.config.get_config()
        dialect = config.get("dialect", None)
        buffer = io.StringIO()

        if dialect is not None:
            writer = csv.writer(buffer, dialect=dialect)
        else:
            writer = csv.writer(
                buffer,
                escapechar=config["escapechar"] or None,
                delimiter=str(config["delimiter"]),
                quotechar=str(config["quotechar"]),
                quoting=int(config["quoting"]),
            )
        processor: "Processor" = self.config.get_processor()
        headers = processor.headers
        if headers:
            writer.writerow(headers)
        for record in queryset:
            values = processor.get_row_values(record)
            writer.writerow(values)
        buffer.seek(0)
        return buffer



import csv
from typing import TYPE_CHECKING
from unittest import mock

import pytest
from strategy_field.utils import fqn

from hope_smart_export.exporters.csv import ExportAsCSV

if TYPE_CHECKING:
    from django.contrib.auth.models import User

    from hope_smart_export.models import Configuration


@pytest.fixture()
def user():
    from demo.factories import UserFactory

    return UserFactory()


@pytest.fixture()
def cfg(db):
    from demo.factories import ConfigurationFactory
    from django.contrib.auth.models import User
    from django.contrib.contenttypes.models import ContentType

    return ConfigurationFactory(
        content_type=ContentType.objects.get_for_model(User),
        columns="username\nemail",
        exporter=fqn(ExportAsCSV),
        data={"delimiter": ",", "quotechar": "'", "quoting": csv.QUOTE_NONE, "escapechar": ""},
    )


@pytest.mark.parametrize("dialect", [None, "excel"])
@pytest.mark.parametrize(
    "line",
    [
        "username\nemail",
        "{{record.username}}\nemail",
        "{{record.username}}\n{{record.email}}",
        "{{record.username}}  \n   {{record.email}}   ",
        "{{record.username}}\n{{record.email}}",
    ],
)
def test_export(db, line, cfg: "Configuration", dialect: str, user: "User"):
    from django.contrib.auth.models import User

    with mock.patch.object(cfg, "columns", f"#comment\n{line}"):
        cfg.data["dialect"] = dialect
        assert len(cfg.get_processor().columns) == 2
        data = cfg.export(User.objects.all())
        assert data.read() == f"""{user.username},{user.email}\r\n"""


def test_export_headers(cfg: "Configuration", user: "User"):
    from django.contrib.auth.models import User

    with mock.patch.object(cfg, "headers", "username\nemail"):
        assert len(cfg.get_processor().columns) == 2
        data = cfg.export(User.objects.all())
        assert data.read() == f"""username,email\r\n{user.username},{user.email}\r\n"""



from django.core.paginator import Paginator

DEFAULT_BATCH_SIZE = 100000


def chunked_iterator(queryset, chunk_size=DEFAULT_BATCH_SIZE):
    paginator = Paginator(queryset, chunk_size)
    for page in range(1, paginator.num_pages + 1):
        for obj in paginator.page(page).object_list:
            yield obj



from unittest import mock

import pytest
from strategy_field.utils import fqn

from hope_smart_export.exporters import ExportAsText


@pytest.fixture()
def user():
    from demo.factories import UserFactory

    return UserFactory()


@pytest.fixture()
def cfg():
    from demo.factories import ConfigurationFactory
    from django.contrib.auth.models import User
    from django.contrib.contenttypes.models import ContentType

    return ConfigurationFactory(
        content_type=ContentType.objects.get_for_model(User),
        columns="username\nemail",
        exporter=fqn(ExportAsText),
        data={"field_separator": ";"},
    )


@pytest.mark.parametrize(
    "line",
    [
        "username\nemail",
        "{{record.username}}\nemail",
        "{{record.username}}\n{{record.email}}",
        "{{record.username}}  \n   {{record.email}}   ",
        "{{record.username}}\n{{record.email}}",
    ],
)
def test_export(db, line, cfg, user):
    from django.contrib.auth.models import User

    with mock.patch.object(cfg, "columns", f"#comment\n{line}"):
        assert len(cfg.get_processor().columns) == 2
        data = cfg.export(User.objects.all())
        assert data.read() == f"""{user.username};{user.email};\n"""


@pytest.mark.parametrize(
    "line",
    [
        "name\ncodename\ncontent_type.app_label",
    ],
)
def test_export_fk(db, line, cfg):
    from django.contrib.auth.models import Permission

    with mock.patch.object(cfg, "columns", f"#comment\n{line}"):
        data = cfg.export(
            Permission.objects.filter(
                content_type__app_label="admin",
                codename="add_logentry",
            ),
        )
    assert data.readline() == "Can add log entry;add_logentry;admin;\n"


@pytest.mark.parametrize(
    "line",
    [
        "name\ncodename\ncontent_type.app_label|upper",
    ],
)
def test_export_filter(db, line, cfg, user):
    from django.contrib.auth.models import Permission

    with mock.patch.object(cfg, "columns", f"#comment\n{line}"):
        data = cfg.export(
            Permission.objects.filter(
                content_type__app_label="admin",
                codename="add_logentry",
            ),
        )
    assert data.readline() == "Can add log entry;add_logentry;ADMIN;\n"


@pytest.mark.parametrize(
    "line",
    [
        "name\n{% for p in record.permissions.all %}{{p.codename}}{%endfor%}",
    ],
)
def test_export_iter(db, line, cfg):
    from demo.factories import GroupFactory
    from django.contrib.auth.models import Group

    GroupFactory(name="Group #1", permissions=["admin.add_logentry"])

    with mock.patch.object(cfg, "columns", f"#comment\n{line}"):
        data = cfg.export(Group.objects.all())
    assert data.readline() == "Group #1;add_logentry;\n"


@pytest.mark.parametrize(
    "headers",
    [
        "1",
        "1\n2",
        "1\n2\n3",
    ],
)
def test_export_balance_headers(db, headers, cfg):
    with mock.patch.object(cfg, "headers", headers):
        assert len(cfg.get_processor().headers) == 2



import pytest
from demo.factories import user_grant_permission
from django.urls import reverse

pytestmark = [pytest.mark.admin]


@pytest.fixture()
def cfg():
    from demo.factories import ConfigurationFactory

    return ConfigurationFactory(columns="id", data={"field_separator": ","})


def test_changelist(django_app, std_user, cfg):
    url = reverse("admin:hope_smart_export_configuration_changelist")

    with user_grant_permission(std_user, ["hope_smart_export.view_configuration"]):
        assert std_user.has_perm("hope_smart_export.view_configuration")
        res = django_app.get(url, user=std_user)
    assert res.status_code == 200


def test_change(django_app, std_user, cfg):
    url = reverse("admin:hope_smart_export_configuration_change", args=(cfg.id,))
    with user_grant_permission(std_user, ["hope_smart_export.change_configuration"]):
        res = django_app.get(url, user=std_user)
    assert res.status_code == 200


def test_add(django_app, std_user, cfg):
    url = reverse("admin:hope_smart_export_configuration_add")
    with user_grant_permission(std_user, ["hope_smart_export.add_configuration"]):
        res = django_app.get(url, user=std_user)
    assert res.status_code == 200


def test_test(django_app, std_user, cfg):
    url = reverse("admin:hope_smart_export_configuration_change", args=(cfg.id,))
    with user_grant_permission(std_user, ["hope_smart_export.change_configuration"]):
        res = django_app.get(url, user=std_user)
        res = res.click("Test")
        res.forms["action-form"]["max_records"] = "-1"
        res = res.forms["action-form"].submit(expect_errors=True)
        assert res.status_code == 400
        res.forms["action-form"]["max_records"] = "10"
        res = res.forms["action-form"].submit()
        assert res.status_code == 200


def test_configure(django_app, std_user, cfg):
    url = reverse("admin:hope_smart_export_configuration_change", args=(cfg.id,))
    with user_grant_permission(std_user, ["hope_smart_export.change_configuration"]):
        res = django_app.get(url, user=std_user)
        res = res.click("Configure")
        res = res.forms["config-form"].submit(expect_errors=True)
        assert res.status_code == 400
        res.forms["config-form"]["field_separator"] = ";"
        res = res.forms["config-form"].submit()
        assert res.status_code == 302



import io

from django import forms
from django.db.models import Model, QuerySet
from django.utils.encoding import force_str

from .base import Exporter, ExporterConfig


class XlsExporterConfig(ExporterConfig):
    defaults = {"sheet_name": "Sheet #1"}
    sheet_name = forms.CharField(max_length=100, required=False)


class ExportAsXls(Exporter):
    config_class = XlsExporterConfig

    def export(self, queryset: QuerySet[Model]) -> io.BytesIO | io.StringIO:
        import xlsxwriter

        config = self.config.get_config()
        out = io.BytesIO()
        book = xlsxwriter.Workbook(out, {"in_memory": True})
        sheet_name = config.pop("sheet_name")
        sheet = book.add_worksheet(sheet_name)
        # TODO: this should be imporved to get fk/nested field types
        formats = {"_general_": book.add_format()}

        row = 0
        sheet.write(row, 0, force_str("#"), formats["_general_"])
        processor = self.config.get_processor()
        headers = processor.headers
        if headers:
            for col, fieldname in enumerate(headers, start=1):
                sheet.write(row, col, force_str(fieldname), formats["_general_"])

        # settingstime_zone = get_default_timezone()
        for rownum, row in enumerate(queryset):
            sheet.write(rownum + 1, 0, rownum + 1)
            values = processor.get_row_values(row)
            for idx, value in enumerate(values):
                fmt = formats.get(idx, formats["_general_"])
                sheet.write(rownum + 1, idx + 1, values[idx], fmt)

        book.close()
        out.seek(0)
        return out



from unittest import mock

import pytest
from django.core.exceptions import ValidationError
from strategy_field.utils import fqn

from hope_smart_export.exporters.txt import ExportAsText


@pytest.fixture()
def cfg(db):
    from demo.factories import ConfigurationFactory
    from django.contrib.auth.models import User
    from django.contrib.contenttypes.models import ContentType

    return ConfigurationFactory(
        content_type=ContentType.objects.get_for_model(User),
        columns="id",
        exporter=fqn(ExportAsText),
        data={"field_separator": ";"},
    )


@pytest.mark.parametrize(
    "line,expected",
    [
        ("username\nemail", True),
        ("{{record.username}}  \nemail ", True),
        ("{{record.username}}\n{{record.email}}", True),
        ("{record.username}}\n{{record.email}}", False),
        ("{{record.username}}\n\n\n{{record.email}}", False),
        ("{$ecord.username}}\n{{record.email}}", False),
        ("{% for %}\n{{record.email}}", False),
        ("123", True),
    ],
)
def test_validate(db, cfg, line, expected):
    with mock.patch.object(cfg, "columns", f"#comment\n{line}"):
        if expected:
            cfg.clean()
        else:
            with pytest.raises(ValidationError):
                cfg.clean()


@pytest.mark.parametrize("opt", ["T", "M"])
def test_process_qs(db, opt, cfg):
    from demo.factories import UserFactory
    from django.contrib.auth.models import User

    with mock.patch.object(cfg, "columns", "username\npassword"):
        with mock.patch.object(cfg, "option", opt):
            with mock.patch("hope_smart_export.utils.DEFAULT_BATCH_SIZE", 10):
                UserFactory.create_batch(20)
                data = cfg.export(User.objects.all())
                assert data


def test_inspect_qs(cfg):
    from django.contrib.auth.models import Permission

    with mock.patch.object(cfg, "columns", "name"):
        qs = Permission.objects.all()[:10]
        data = cfg.inspect(qs, 5)
        assert len(data.captured_queries) == 1

    with mock.patch.object(cfg, "columns", "content_type.model"):
        data = cfg.inspect(qs)
        assert len(data.captured_queries) == 10
